#ifndef UIMANAGER_H_
#define UIMANAGER_H_

#include <QObject>
#include <QList>

#include "Meeting.h"
#include "DeviceManager.h"
#include "PasswordDialog.h"

class Engine;
class WindowManager;
class WeeklyViewWidget;
class SettingsView;
class RoomStatusIndicatorWidget;
class ProgressBar;
class MeetingInfoDialog;
class CommunicationManager;
class QDateTime;

class UIManager : public QObject
{
	Q_OBJECT
	
public:
	UIManager( Engine *aEngine, WindowManager *aWindowManager );
	virtual ~UIManager();
	
	void connectDeviceManager( DeviceManager *aDeviceManager );
	void connectCommunicationManager( CommunicationManager *aCommunicationManager );
	void showMainView();
	void showProgressBar( QString aText );

signals:
	
public slots:

	void settingsViewRequest();
	void settingsOkClicked();
	void roomStatusIndicatorRequested();
	void previousViewRestored();
	void changeModeOrdered( DeviceManager::OperationMode aMode );
	
private slots:

	void meetingsFetched( const QList<Meeting*> &aMeetings );
	void meetingDetailsFetched( Meeting &aDetailedMeeting );
	void currentRoomChanged( Room *aRoom );
	void progressBarCancelled();
	void updateTime( QDateTime aDateTime );
	void passwordEntered( PasswordDialog::PasswordStatus aStatus );
	void showMeetingProgressBar( Meeting *aMeeting );
	void updateProgressBarText( const QString &aText );
	void hideProgressBar();

private:
	
	void createWeeklyView();
	void createSettingsView();
	void createRoomStatusIndicator();
	void createPasswordDialog();
	void createProgressBar();
	void createMeetingInfoDialog();
	
private:
	Engine *iEngine;
	WindowManager *iWindowManager;
	
	WeeklyViewWidget *iWeeklyView;
	SettingsView *iSettingsView;
	RoomStatusIndicatorWidget *iRoomStatusIndicator;
	PasswordDialog *iPasswordDialog;
	ProgressBar *iProgressBar;
	MeetingInfoDialog *iMeetingInfo;
};

#endif /*UIMANAGER_H_*/
